import jDisco.Process;
import jDisco.Random;
import jDisco.Reporter;
import jDisco.Variable;

import javax.swing.*;
import java.awt.*;
import static java.lang.Math.*;

public class NBodySimulation extends Process {
    static int MAX_N = 15;
    static final double SCALE = 100.0, VSCALE = 200.0;
    static final double G = 9.81, RMIN = 40.0;
    int N;
    Variable[] x, y, vx, vy;
    double[] m, r;
    Color color[];
    Random rand = new Random();
    JFrame frame = new JFrame();

    NBodyProblem() {
        integrationMethod("RKV56");
        frame.setSize(600, 600);
        frame.getContentPane().add(new Canvas());
        frame.setVisible(true);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        N = 2 + rand.nextInt(MAX_N - 2);
        x = new Variable[N];
        y = new Variable[N];
        vx = new Variable[N];
        vy = new Variable[N];
        m = new double[N];
        r = new double[N];
        color = new Color[N];
        double Sx = 0, Sy = 0, Svx = 0, Svy = 0, Sm = 0;
        for (int i = 0; i < N; i++) {
            x[i] = new Variable(SCALE * rand.nextGaussian()).start();
            y[i] = new Variable(SCALE * rand.nextGaussian()).start();
            vx[i] = new Variable(VSCALE * rand.nextGaussian()).start();
            vy[i] = new Variable(VSCALE * rand.nextGaussian()).start();
            r[i] = MAX_N + 30 * Math.random();
            m[i] = r[i] * r[i] * r[i];
            Sx += m[i] * x[i].state;
            Sy += m[i] * y[i].state;
            Svx += m[i] * vx[i].state;
            Svy += m[i] * vy[i].state;
            Sm += m[i];
        }
        for (int i = 0; i < N; i++) {
            x[i].state -= Sx / Sm - frame.getSize().width / 2;
            y[i].state -= Sy / Sm - frame.getSize().height / 2;
            vx[i].state -= Svx / Sm;
            vy[i].state -= Svy / Sm;
            color[i] = new Color(abs(rand.nextInt()) % 256,
                    abs(rand.nextInt()) % 256,
                    abs(rand.nextInt()) % 256);
        }
    }

    public void derivatives() {
        for (int i = 0; i < N; i++) {
            x[i].rate = vx[i].state;
            y[i].rate = vy[i].state;
            vx[i].rate = vy[i].rate = 0;
            for (int j = 0; j < N; j++) {
                if (j != i) {
                    double R = sqrt(pow(x[i].state - x[j].state, 2) +
                            pow(y[i].state - y[j].state, 2));
                    double Rm = max(RMIN, R);
                    vx[i].rate += G * m[j] * (x[j].state - x[i].state) / 
                                  (Rm * Rm);
                    vy[i].rate += G * m[j] * (y[j].state - y[i].state) / 
                                  (Rm * Rm);
                }
            }
        }
    }

    class Canvas extends JComponent {
        public void paint(Graphics g) {
            g.setColor(Color.white);
            Dimension d = getSize();
            g.fillRect(0, 0, d.width, d.height);
            for (int i = 0; i < N; i++) {
                int ix = (int) round(x[i].state - r[i] / 2),
                        iy = (int) round(y[i].state - r[i] / 2),
                        ir = (int) round(r[i]);
                g.setColor(color[i]);
                g.fillOval(ix, iy, ir, ir);
            }
        }
    }

    public void actions() {
        dtMin = 1e-10;
        dtMax = 0.01;
        maxRelError = 1e-1;
        start();
        new Reporter() {
            public void actions() {
                Graphics g = frame.getGraphics();
                frame.update(g);
            }
        }.setFrequency(0.0001).start();
        hold(10000);
    }

    public static void main(String[] args) {
        activate(new NBodyProblem());
    }
}

