import jDisco.*;
import jDisco.Process;

/*
 * Numerical solution of the nonlinear partial differential equation 
 * on the interval [0,1] given by
 *
 *     du/dt = -u * du/dx + x * d/dx(u*du/dx) + t
 *
 * Initial and boundary conditions:
 *
 *     u(0,x) = x, u(t, 0) = 50, du(t,1)/dt = 1 - sin(u(t,1))
 */ 

public class PDEExample extends Process {
    PDEVariable u;
        
    class U extends PDEVariable {
        U(double xLeft, double xRight, int nPoints) {
            super(xLeft, xRight, nPoints);
        }
        
        public double rate() { 
            return -state() * dx() + x() * ddx() + t(); 
        }
        
        public double D() { return state(); } 
        
        public double initialState() { return x(); }
        
        public double b1Left()  { return 1; }
        public double b3Left()  { return 50; }
        public double b1Right() { return 1; }
        public double b3Right() { return 1 - Math.sin(state()); }
    } 
    
     public void actions() {
         dtMin = 1.0e-5; dtMax = 1.0; 
         maxAbsError = maxRelError = 1.0e-4;
         u = new U(0, 1, 21);
         u.start();
         hold(1.2);
         u.show(" U ");
    }
        
    public static void main(String[] args) {
        activate(new PDEExample());
    }
}
