import jDisco.*;
import jDisco.Process;

/*
 * Simulation of a spring supporting a mass with a viscous damper
 * attached.
 *
 * The model has been taken from
 *
 *     Advanced Continuous Simulation Language (ACSL)
 *     Reference Manual, 1999, (p. A-5 - A-8)
 *     http://www.ACSL.sim.com
 */

public class Spring extends Process {
    static final double w = 1.0, a = 1.0, g = 32.2, k = 0.02;
    Variable x, xd;
    
    public void derivatives() {
        xd.rate = (w - k * xd.state - a * x.state) / (w / g);
        x.rate = xd.state; 
    } 
    
     public void actions() {
         x = new Variable(0).start();
         xd = new Variable(0).start();
         start(); 
         final Graph graph = new Graph(" x / xd ");
         new Reporter() {
            public void actions() {
                graph.add(xd, x);
            }
        }.setFrequency(0.02).start();
        hold(4);
        graph.show();
    }
    
    public static void main(String[] args) {
        long startTime = System.currentTimeMillis();
        activate(new Spring());
        long stopTime = System.currentTimeMillis();
        System.out.println("Time used = " + 
            (stopTime - startTime) / 1000.0 + " seconds");
    }
}


