import jDisco.*;
import jDisco.Process;

/*
 * Simulation of interactions between population, industrial growth, 
 * food production and limits in the ecosystems of the Earth.
 *
 * The model has been documented in the book
 *
 *      D. L. Meadows et. al.,
 *      Dynamics of Growth in a Finite World.
 *      Wright-Allen, 1974.
 *
 * As far as possible the book's nomenclature has been followed.
 * The table data for the simulation may be found in file World3data.
 */

import java.util.*;
import java.io.*;
import static java.lang.Math.*;

public class World3 extends Process {
    double AHL, AHLM, AHL70, AII, AIPH, ALAI, ALAI1, ALAI2, ALI, ALIC,
        ALIC1, ALIC2, ALL, ALLN, ALPHA, ALSC, ALSC1, ALSC2, AMTI, B,
        CBR, CDR, CMI, CMPLE, COPM, CUF, D, DCFS, DCFSN, DCPH, DNRUR,
        DPOLX, DTF, D1, D2, D3, D4, F, FALM, FCAOR, FCAOR1, FCAOR2,
        FCE, FCEST, FIALD, FIE, FIOAA, FIOAA1, FIOAA2, FIOAC, FIOACC,
        FIOACV, FIOAC1, FIOAC2, FIOAI, FIOAS, FIOAS1, FIOAS2, FIPM,
        FM, FOA, FOI, FOS, FPC, FPU, FRPM, FRSN, FSAFC, FSPD, HSID,
        ICDR, ICI, ICIR, ICOR, ICOR1, ICOR2, IEAT, IET, IFPC, IFPC1,
        IFPC2, ILF, IMEF, IMTI, IO, IO70, IOPCD, ISOPC, ISOPC1, 
        ISOPC2, J, JPH, JPICU, JPSCU, LDR, LEN, LER, LF, LFC, LFD, 
        LFDR, LFERTI, LFH, LFPF, LFR, LFRT, LLMY, LLMY1, LLMY2, LMC, 
        LMF, LMHS, LMHS1, LMHS2, LMP, LPD, LRUI, LUFDT, LY, LYCM, LYF, 
        LYF1, LYF2, LYMAP, LYMAP1, LYMAP2, LYMC, LYTD, MAT1, MAT2, 
        MAT3, MAT4, MLYMC, MPAI, MPLD, MTF, MTFN, M1, M2, M3, M4, NFC,
        NRFR, NRI, NRUF, NRUF1, NRUF2, NRUR, PALI, PALT, PCRUM, PET,
        PJAS, PJIS, PJSS, PL, POP, PPGAO, PPGF, PPGF1, PPGF2, PPGIO, 
        PPOLI, PPOLX, PPOL70, PPTD, PPASR, PYEAR, P1I, P2I, P3I, P4I,
        RLT, SAD, SCDR, SCI, SCIR, SCOR, SCOR1, SCOR2, SD, SFPC, SFSN, 
        SO, SOPC, TAI, TF, UILDT, UILI, UILPC, UILR, ZPGT;

    Table AHLMT, CMIT, CMPLET, CUFT, DCPHT, FALMT, FCAOR1T, FCAOR2T, 
        FCET, FIALDT, FIOAA1T, FIOAA2T, FIOACVT, FIOAS1T, FIOAS2T, 
        FMT, FPUT, FRSNT, FSAFCT, HSAPCT, IFPC1T, IFPC2T, ISOPC1T, 
        ISOPC2T, JPHT, JPICUT, JPSCUT, LFDRT, LFRTT, LLMY1T, LLMY2T, 
        LMFT, LMHS1T, LMHS2T, LMPT, LYMAP1T, LYMAP2T, LYMCT, MLYMCT, 
        M1T, M2T, M3T, M4T, PCRUMT, SFSNT, UILPCT;

    Variable AL, IC, LFERT, NR, PAL, PPOL, P1, P2, P3, P4, SC, UIL;
    Variable HSAPC, LEZ, IOPC, LUF, CAI, FR, PPGR, FCAPC;
    Smooth EHSPC, AIOPC, LUFD, AI, PFR;
    ExpDelay PLE, DIOPC, PPAPR, FCFPC;

    Scanner in;
    Graph POPGraph;

     World3() {
        initialize();
        try {
            in = new Scanner(new FileReader("World3data"));
        } catch(IOException e) {}
        POPGraph = new Graph("Population");   
        POPGraph.setYMin(0);
    }

    public void actions() {
        reactivate(this, at, 1900);
        dtMin = 1.0 / 365 / 24;
        dtMax = 1;
        maxRelError = 1e-5;
        M1T = read(20, 80, 10);
        M2T = read(20, 80, 10);
        M3T = read(20, 80, 10);
        M4T = read(20, 80, 10);
        LMFT = read(0, 5, 1);
        HSAPCT = read(0, 2000, 250);
        LMHS1T = read(0, 100, 20);
        LMHS2T = read(0, 100, 20);
        FPUT = read(0, 16e9, 2e9);
        CMIT = read(0, 1600, 200);
        LMPT = read(0, 100, 10);
        FMT = read(0, 80, 10);
        CMPLET = read(0, 80, 10);
        SFSNT = read(0, 800, 200);
        FRSNT = read(-0.2, 0.2, 0.1);
        FCET = read(0, 3, 0.5);
        FSAFCT = read(0, 10, 2);
        FIOACVT = read(0, 2, 0.2);
        ISOPC1T = read(0, 1600, 200);
        ISOPC2T = read(0, 1600, 200);
        FIOAS1T = read(0, 2, 0.5);
        FIOAS2T = read(0, 2, 0.5);
        JPICUT = read(50, 800, 150);
        JPSCUT = read(50, 800, 150);
        JPHT = read(2, 30, 4);
        CUFT = read(1, 11, 2);
        IFPC1T = read(0, 1600, 200);
        IFPC2T = read(0, 1600, 200);
        FIOAA1T = read(0, 2.5, 0.5);
        FIOAA2T = read(0, 2.5, 0.5);
        DCPHT = read(0, 1, 0.1);
        LYMCT = read(0, 1000, 40);
        LYMAP1T = read(0, 30, 10);
        LYMAP2T = read(0, 30, 10);
        FIALDT = read(0, 2, 0.25);
        MLYMCT = read(0, 600, 40);
        LLMY1T = read(0, 9, 1);
        LLMY2T = read(0, 9, 1);
        UILPCT = read(0, 1600, 200);
        LFDRT = read(0, 30, 10);
        LFRTT = read(0, 0.10, 0.02);
        FALMT = read(0, 4, 1);
        PCRUMT = read(0, 1600, 200);
        FCAOR1T = read(0, 1, 0.1);
        FCAOR2T = read(0, 1, 0.1);
        AHLMT = read(1, 1001, 250);

        AL = new Variable(ALI);
        AL.start();
        IC = new Variable(ICI);
        IC.start();
        LFERT = new Variable(LFERTI);
        LFERT.start();
        NR = new Variable(NRI);
        NR.start();
        PAL = new Variable(PALI);
        PAL.start();
        PPOL = new Variable(PPOLI);
        PPOL.start();
        P1 = new Variable(P1I);
        P1.start();
        P2 = new Variable(P2I);
        P2.start();
        P3 = new Variable(P3I);
        P3.start();
        P4 = new Variable(P4I);
        P4.start();
        SC = new Variable(SCI);
        SC.start();
        UIL = new Variable(UILI);
        UIL.start();

        HSAPC = new Variable(0);
        LEZ = new Variable(0);
        IOPC = new Variable(0);
        LUF = new Variable(0);
        CAI = new Variable(0);
        FR = new Variable(0);
        PPGR = new Variable(0);
        FCAPC = new Variable(0);

        start();
        pause();
        AI = new Smooth(CAI, ALAI1);
        AI.start();
        PFR = new Smooth(FR, FSPD);
        PFR.start();
        EHSPC = new Smooth(HSAPC, HSID);
        EHSPC.start();
        AIOPC = new Smooth(IOPC, IEAT);
        AIOPC.start();
        LUFD = new Smooth(LUF, LUFDT);
        LUFD.start();
        PLE = new ExpDelay(3, LEZ.state, LEZ, LPD);
        PLE.start();
        DIOPC = new ExpDelay(3, IOPC.state, IOPC, SAD);
        DIOPC.start();
        PPAPR = new ExpDelay(3, PPGR.state, PPGR, PPTD);
        PPAPR.start();
        FCFPC = new ExpDelay(3, FCAPC.state, FCAPC, PPTD);
        FCFPC.start();

        new Reporter() {
            public void actions() {
                POPGraph.add(POP);
            }
        }.setFrequency(5).start();
        reactivate(this, at, 2100);
        POPGraph.show();
    }

    public void derivatives() {
        POP = P1.state + P2.state + P3.state + P4.state;         //   1 
        PPOLX = PPOL.state / PPOL70;                             // 143
        AHLM = AHLMT.value(PPOLX);                               // 145
        AHL = AHL70 * AHLM;                                      // 146
        PPASR = PPOL.state / (AHL * 1.4);                        // 144
        NRFR = NR.state / NRI;                                   // 133
        FCAOR1 = FCAOR1T.value(NRFR);                            // 135
        FCAOR2 = FCAOR2T.value(NRFR);                            // 136
        FCAOR = time() > PYEAR ? FCAOR2 : FCAOR1;                // 134
        FALM = PFR != null ? FALMT.value(PFR.state())            // 126
            : FALMT.value(FR.state);
        AIPH = AI != null ? AI.state() * (1 - FALM) / AL.state   // 101
            : AII * (1 - FALM) / AL.state;
        JPH = JPHT.value(AIPH);                                  //  79
        PJAS = JPH * AL.state;                                   //  78
        CUF = LUFD != null ? CUFT.value(LUFD.state()) : 1;       //  83
        SCOR = time() > PYEAR ? SCOR2 : SCOR1;                   //  72
        SO = SC.state * CUF / SCOR;                              //  70 
        SOPC = SO / POP;                                         //  71
        JPSCU = JPSCUT.value(SOPC) * 1e-3;                       //  77
        PJSS = SC.state * JPSCU;                                 //  76
        ICOR = time() > PYEAR ? ICOR2 : ICOR1;                   //  51
        ALIC = time() > PYEAR ? ALIC2 : ALIC1;                   //  54
        ALSC = time() > PYEAR ? ALSC2 : ALSC1;                   //  69
        ICDR = IC.state / ALIC;                                  //  53
        SCDR = SC.state / ALSC;                                  //  68
        IO = IC.state * (1 - FCAOR) * CUF / ICOR;                //  50
        IOPC.state = IO / POP;                                   //  49
        JPICU = JPICUT.value(IOPC.state) * 1e-3;                 //  75
        PJIS = IC.state * JPICU;                                 //  74
        J = PJIS + PJAS + PJSS;                                  //  73
        LF = (P2.state + P3.state) * LFPF;                       //  80
        LUF.state = J / LF;                                      //  81
        PCRUM = PCRUMT.value(IOPC.state);                        // 132
        PPGF = time() > PYEAR ? PPGF2 : PPGF1;                   // 138
        PPGIO = PCRUM * POP * FRPM * IMEF * IMTI;                // 139
        PPGAO = AIPH * AL.state * FIPM * AMTI;                   // 140
        PPGR.state = (PPGIO + PPGAO) * PPGF;                     // 137
        NRUF = time() > PYEAR ? NRUF2 : NRUF1;                   // 131
        NRUR = POP * PCRUM * NRUF;                               // 130
        LFDR = LFDRT.value(PPOLX);                               // 122
        LFD = LFERT.state * LFDR;                                // 123
        LFRT = LFRTT.value(FALM);                                // 125
        LFR = (ILF - LFERT.state) / LFRT;                        // 124
        UILPC = UILPCT.value(IOPC.state);                        // 117
        UILR = UILPC * POP;                                      // 118
        LRUI = max(0, (UILR - UIL.state) / UILDT);               // 119
        LYMAP1 = LYMAP1T.value(IO / IO70);                       // 106
        LYMAP2 = LYMAP2T.value(IO / IO70);                       // 107
        LYMAP = time() > PYEAR ? LYMAP2 : LYMAP1;                // 105
        LYMC = LYMCT.value(AIPH);                                // 102
        LYF = time() > PYEAR ? LYF2 : LYF1;                      // 104
        LY = LYF * LFERT.state * LYMC * LYMAP;                   // 103
        LLMY1 = LLMY1T.value(LY / ILF);                          // 114
        LLMY2 = LLMY2T.value(LY / ILF);                          // 115
        LLMY = time() > PYEAR ? LLMY2 : LLMY1;                   // 113
        ALL = ALLN * LLMY;                                       // 112
        LER = AL.state / ALL;                                    // 116
        MLYMC = MLYMCT.value(AIPH);                              // 111
        DCPH = DCPHT.value(PAL.state / PALT);                    //  97
        MPLD = LY / (DCPH * SD);                                 // 109
        ALAI = time() > PYEAR ? ALAI2 : ALAI1;                   // 100
        MPAI = ALAI * LY * MLYMC / LYMC;                         // 110
        FIALD = FIALDT.value(MPLD / MPAI);                       // 108
        F = LY * AL.state * LFH * (1 - PL);                      //  87 
        FPC = F / POP;                                           //  88
        FR.state = FPC / SFPC;                                   // 127
        IFPC1 = IFPC1T.value(IOPC.state);                        //  90
        IFPC2 = IFPC2T.value(IOPC.state);                        //  91
        IFPC = time() > PYEAR ? IFPC2 : IFPC1;                   //  89
        FIOAA1 = FIOAA1T.value(FPC / IFPC);                      //  94
        FIOAA2 = FIOAA2T.value(FPC / IFPC);                      //  95
        FIOAA = time() > PYEAR ? FIOAA2 : FIOAA1;                //  93
        TAI = IO * FIOAA;                                        //  92 
        CAI.state = TAI * (1 - FIALD);                           //  98
        LDR = TAI * FIALD / DCPH;                                //  96
        LFC = AL.state / PALT;                                   //  84
        ISOPC1 = ISOPC1T.value(IOPC.state);                      //  61
        ISOPC2 = ISOPC2T.value(IOPC.state);                      //  62
        ISOPC = time() > PYEAR ? ISOPC2 : ISOPC1;                //  60
        FIOAS1 = FIOAS1T.value(SOPC / ISOPC);                    //  64
        FIOAS2 = FIOAS2T.value(SOPC / ISOPC);                    //  65
        FIOAS = time() > PYEAR ? FIOAS2 : FIOAS1;                //  63
        SCIR = IO * FIOAS;                                       //  66
        FIOACC = time() > PYEAR ? FIOAC2 : FIOAC1;               //  58
        FIOACV = FIOACVT.value(IOPC.state / IOPCD);              //  59 
        FIOAC = time() > IET ? FIOACV : FIOACC;                  //  57
        FIOAI = 1 - FIOAA - FIOAS - FIOAC;                       //  56
        ICIR = IO * FIOAI;                                       //  55
        HSAPC.state = HSAPCT.value(SOPC);                        //  21 
        LMHS1 = EHSPC != null ? LMHS1T.value(EHSPC.state())      //  24
            : LMHS1T.value(HSAPC.state);
        LMHS2 = EHSPC != null ? LMHS2T.value(EHSPC.state())      //  25
            : LMHS2T.value(HSAPC.state);
        LMHS = time() > 1940 ? LMHS2 : LMHS1;                    //  23
        FPU = FPUT.value(POP);                                   //  26
        CMI = CMIT.value(IOPC.state);                            //  27
        LMC = 1 - CMI * FPU;                                     //  28
        LMP = LMPT.value(PPOLX);                                 //  29
        LMF = LMFT.value(FPC / SFPC);                            //  20 
        LEZ.state = LEN * LMF * LMHS * LMP * LMC;                //  19
        FM = FMT.value(LEZ.state);                               //  34 
        MTF = MTFN * FM;                                         //  33
        SFSN = DIOPC != null ? SFSNT.value(DIOPC.state)          //  39
            : SFSNT.value(IOPC.state);
        FIE = AIOPC != null ? 
              (IOPC.state - AIOPC.state()) / AIOPC.state() : 0;  //  42
        FRSN = FRSNT.value(FIE);                                 //  41
        DCFS = time() > ZPGT ? 2.0 : DCFSN * FRSN * SFSN;        //  38
        CMPLE = PLE != null ? CMPLET.value(PLE.state)            //  36
            : CMPLET.value(LEZ.state);
        DTF = DCFS * CMPLE;                                      //  35
        NFC = MTF / DTF - 1;                                     //  44
        FSAFC = FSAFCT.value(NFC);                               //  48
        FCAPC.state = FSAFC * SOPC;                              //  47
        FCE = time() > FCEST ? 1.0 :                             //  45
            FCFPC != null ? FCET.value(FCFPC.state)
            : FCET.value(FCAPC.state);
        TF = min(MTF, MTF * (1 - FCE) + DTF * FCE);              //  32
        M1 = M1T.value(LEZ.state);                               //   4
        D1 = P1.state * M1;                                      //   3
        MAT1 = P1.state * (1 - M1) / 15;                         //   5
        M2 = M2T.value(LEZ.state);                               //   8
        D2 = P2.state * M2;                                      //   7
        MAT2 = P2.state * (1 - M2) / 30;                         //   9
        M3 = M3T.value(LEZ.state);                               //  12
        D3 = P3.state * M3;                                      //  11
        MAT3 = P3.state * (1 - M3) / 20;                         //  13
        M4 = M4T.value(LEZ.state);                               //  16
        D4 = P4.state * M4;                                      //  15
        D = D1 + D2 + D3 + D4;                                   //  17
        B = time() > PET ? D : TF * P2.state * 0.5 / RLT;        //  30
        CBR = 1000 * B / POP;                                    //  31
        CDR = 1000 * D / POP;                                    //  18
        FOA = 0.22 * F / (0.22 * F + SO + IO);                   // 147
        FOI = IO / (0.22 * F + SO + IO);                         // 148
        FOA = SO / F / (0.22 * F + SO + IO);                     // 149

        P1.rate = B - D1 - MAT1;                                 //   2
        P2.rate = MAT1 - D2 - MAT2;                              //   6
        P3.rate = MAT2 - D3 - MAT3;                              //  10
        P4.rate = MAT3 - D4;                                     //  14
        IC.rate = ICIR - ICDR;                                   //  52
        SC.rate = SCIR - SCDR;                                   //  67
        AL.rate = LDR - LER - LRUI;                              //  85
        PAL.rate = -LDR;                                         //  86
        UIL.rate = LRUI;                                         // 120
        LFERT.rate = LFR - LFD;                                  // 121
        NR.rate = -NRUR;                                         // 129
        PPOL.rate = (PPAPR != null ? PPAPR.state                 // 142
                     : PPGR.state) - PPASR;
   }

    private Table read(double xMin, double xMax, double xStep) {
        int N = (int) ((xMax - xMin) / xStep + 1), i;
        Table t = new Table();
        for (i = 1; i <= N; i++)
            t.add(xMin + (i - 1) * xStep, in.nextDouble());
        in.nextLine();
        return t;
    }

    private void initialize() {
        P1I = 65e7;
        P2I = 70e7;
        P3I = 19e7;
        P4I = 6e7;
        LEN = 28;
        HSID = 20;
        RLT = 30;
        PET = 4000;
        MTFN = 12;
        LPD = 20;
        DCFSN = 4;
        ZPGT = 4000;
        SAD = 20;
        FRSN = 0.82;
        IEAT = 3;
        FCEST = 4000;
        ICOR1 = 3;
        ICOR2 = 3;
        ALIC1 = 14;
        ALIC2 = 14;
        IET = 4000;
        FIOAC1 = 0.43;
        FIOAC2 = 0.43;
        IOPCD = 400;
        ICI = 2.1e11;
        SCI = 1.44e11;
        ALSC1 = 20;
        ALSC2 = 20;
        SCOR1 = 1;
        SCOR2 = 1;
        LFPF = 0.75;
        LUFDT = 2;
        CUF = 1;
        PALT = 3.2e9;
        ALI = 0.9e9;
        PALI = 2.3e9;
        LFH = 0.7;
        PL = 0.1;
        AII = 5e9;
        ALAI1 = 2;
        ALAI2 = 2;
        LYF1 = 1;
        LYF2 = 1;
        IO70 = 7.9e11;
        SD = 0.07;
        ALLN = 6000;
        UILDT = 10;
        UILI = 8.2e6;
        LFERTI = 600;
        ILF = 600;
        SFPC = 230;
        FSPD = 2;
        NRI = 1e12;
        NRUF1 = 1;
        NRUF2 = 1;
        PPGF1 = 1;
        PPGF2 = 1;
        FRPM = 0.02;
        IMEF = 0.1;
        IMTI = 10;
        FIPM = 0.001;
        AMTI = 1;
        PPTD = 20;
        PPOLI = 2.5e7;
        PPOL70 = 1.36e8;
        AHL70 = 1.5;
        PYEAR = 1975;
    }

    public static void main(String[]arg) {
        activate(new World3());
    }
}

class Smooth extends Continuous {
    Smooth(Variable target, double lag) {
        this.target = target;
        this.lag = lag;
        r = new Variable(target.state * lag);
    }
    
    public void derivatives() {
        r.rate = target.state - r.state / lag;
    }

    public double state() {
        return r.state / lag;
    }

    public Continuous start() {
        super.start();
        r.start();
        return this;
    }

    public void stop() {
        super.stop();
        r.stop();
    }
    
    private Variable target, r;
    private double lag;
}
